import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { firstValueFrom, forkJoin, lastValueFrom } from 'rxjs';
import { Category } from '../../structure-list/models/category.model';
import { Module } from '../../structure-list/models/module.model';
import { SearchService } from '../../structure-list/services/search.service';
import { SearchQuery } from '../models/searchQuery.model';

@Component({
  selector: 'app-annuaire-header',
  templateUrl: './annuaire-header.component.html',
  styleUrls: ['./annuaire-header.component.scss'],
})
export class AnnuaireHeaderComponent implements OnInit, OnChanges {
  @Input() shouldResetFilters = 0;
  @Output() searchEvent = new EventEmitter<SearchQuery>();

  public jobGroupCategories: Category[] = [];
  public employerCategories: Category[] = [];
  public territoryCategories: Category[] = [];
  public communeCategories: Category[] = [];

  public searchInput = '';
  public checkedModulesFilter: Module[] = [];
  public jobGroupCheckedModules: Module[] = [];
  public employerCheckedModules: Module[] = [];
  public territoryCheckedModules: Module[] = [];
  public communeCheckedModules: Module[] = [];

  constructor(
    private activatedRoute: ActivatedRoute,
    private router: Router,
    public searchService: SearchService,
  ) {}

  async ngOnInit(): Promise<void> {
    // Will store the different categories
    await this.loadFilters();

    this.loadQueryParams();

    if (this.searchService.previousResult$.getValue().docs.length === 0) {
      this.applyFilter();
    } else {
      this.setCheckedModulesFilter();
    }
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.shouldResetFilters?.currentValue !== 0) {
      this.resetFilters();
    }
  }

  private loadQueryParams(): void {
    const queryParams = this.activatedRoute.snapshot.queryParams;
    this.searchInput = queryParams['search'] ?? '';
    const jobGroupFilterChecked = queryParams['jobs']?.split('|') ?? [];
    const employerFilterChecked = queryParams['employers']?.split('|') ?? [];
    const territoryFilterChecked = queryParams['territories']?.split('|') ?? [];
    const communeFilterChecked = queryParams['communes']?.split('|') ?? [];

    // We need to map to set the type in module name, as it is done in checkedModules of the collapsable-filter/more-filters component
    this.jobGroupCheckedModules = this.jobGroupCategories[0]?.modules
      .filter((module) => jobGroupFilterChecked.includes(module.name))
      .map((module) => ({ ...module, displayText: module.name, name: 'jobGroup' }));

    this.employerCheckedModules = this.employerCategories[0]?.modules
      .filter((module) => employerFilterChecked.includes(module.name))
      .map((module) => ({ ...module, displayText: module.name, name: 'employer' }));

    this.territoryCheckedModules = this.territoryCategories[0]?.modules
      .filter((module) => territoryFilterChecked.includes(module.name))
      .map((module) => ({ ...module, displayText: module.name, name: 'ctm' }));

    this.communeCheckedModules = this.communeCategories[0]?.modules
      .filter((module) => communeFilterChecked.includes(module.name))
      .map((module) => ({ ...module, displayText: module.name, name: 'commune' }));
  }

  public onSearchSubmitted(value: string): void {
    this.searchInput = value;
    this.applyFilter();
  }

  private updateQueryParams(): void {
    const queryParams: Record<string, string> = {};

    if (this.searchInput.trim()) {
      queryParams.search = this.searchInput.trim();
    }
    if (this.jobGroupCheckedModules.length) {
      queryParams.jobs = this.jobGroupCheckedModules.map((module) => module.displayText).join('|');
    }
    if (this.employerCheckedModules.length) {
      queryParams.employers = this.employerCheckedModules.map((module) => module.displayText).join('|');
    }
    if (this.territoryCheckedModules.length) {
      queryParams.territories = this.territoryCheckedModules.map((module) => module.displayText).join('|');
    }
    if (this.communeCheckedModules.length) {
      queryParams.communes = this.communeCheckedModules.map((module) => module.displayText).join('|');
    }

    // Delay to avoid error in console: "NG0100: ExpressionChangedAfterItHasBeenCheckedError"
    setTimeout(
      () =>
        this.router.navigate([], {
          relativeTo: this.activatedRoute,
          queryParams: queryParams,
        }),
      1,
    );
  }

  private setCheckedModulesFilter(): void {
    this.checkedModulesFilter = [
      ...this.jobGroupCheckedModules,
      ...this.employerCheckedModules,
      ...this.territoryCheckedModules,
      ...this.communeCheckedModules,
    ];
  }

  private applyFilter(page = 1): void {
    this.setCheckedModulesFilter();

    this.searchEvent.emit({
      search: this.searchInput.trim(),
      page: page,
      jobFilters: this.jobGroupCheckedModules.map((module) => module.id),
      employerFilters: this.employerCheckedModules.map((module) => module.id),
      territoryFilters: this.territoryCheckedModules.map((module) => module.id),
      communeFilters: this.communeCheckedModules.map((module) => module.id),
    });

    this.updateQueryParams();
  }

  /** Get the categories for each modal type */
  private async loadFilters(): Promise<void> {
    const { jobGroups, employers } = await lastValueFrom(
      forkJoin({
        jobGroups: this.searchService.getJobsGroups(),
        employers: this.searchService.getEmployers(),
      }),
    );
    this.jobGroupCategories = [
      {
        id: 'jobGroup',
        name: 'Fonction',
        theme: 'Fonction',
        modules: [],
      },
    ];
    jobGroups.forEach((jobGroup) => {
      this.jobGroupCategories[0].modules.push({
        disabled: false,
        name: jobGroup.name,
        id: jobGroup._id,
      });
    });

    this.employerCategories = [
      {
        id: 'employer',
        name: 'Employeur',
        theme: 'Employeur',
        modules: [],
      },
    ];
    employers.forEach((employer) => {
      this.employerCategories[0].modules.push({
        disabled: false,
        name: employer.name,
        id: employer._id,
      });
    });

    // Territories
    const categories = await firstValueFrom(this.searchService.getCategories());
    const ctmCategory = categories.find((c) => c.id === 'ctm');
    if (ctmCategory) {
      this.territoryCategories = [
        {
          id: 'ctm',
          name: 'ctm',
          theme: 'ctm',
          modules: ctmCategory.modules.map((module) => ({
            disabled: false,
            name: module.name,
            id: module.id,
          })),
        },
      ];
    }

    // Communes
    const communes = await lastValueFrom(this.searchService.getCommunesWithUsers());
    this.communeCategories = [
      {
        id: 'commune',
        name: 'Commune',
        theme: 'Communes',
        modules: communes.map((commune) => ({
          disabled: false,
          name: commune.commune,
          id: commune.inseeCode,
        })),
      },
    ];
  }

  public onSelectJobGroups(jobGroupCheckedModules: Module[]): void {
    this.jobGroupCheckedModules = jobGroupCheckedModules;
    this.applyFilter();
  }

  public onSelectEmployers(employerCheckedModules: Module[]): void {
    this.employerCheckedModules = employerCheckedModules;
    this.applyFilter();
  }

  public onSelectTerritory(territoryCheckedModules: Module[]): void {
    this.territoryCheckedModules = territoryCheckedModules;
    this.applyFilter();
  }

  public onSelectCommune(communeCheckedModules: Module[]): void {
    this.communeCheckedModules = communeCheckedModules;
    this.applyFilter();
  }

  public resetFilters(): void {
    this.searchInput = '';
    this.jobGroupCheckedModules = [];
    this.employerCheckedModules = [];
    this.territoryCheckedModules = [];
    this.communeCheckedModules = [];
    this.applyFilter();
  }

  public removeFilter(module: Module): void {
    const index: number = this.checkedModulesFilter.findIndex((m: Module) => m.id === module.id);
    if (index !== -1) {
      // update global list of checked filters
      this.checkedModulesFilter = this.checkedModulesFilter.filter((m: Module) => m.id !== module.id);
      // update each select
      this.jobGroupCheckedModules = this.checkedModulesFilter.filter((module) => module.name === 'jobGroup');
      this.employerCheckedModules = this.checkedModulesFilter.filter((module) => module.name === 'employer');
      this.territoryCheckedModules = this.checkedModulesFilter.filter((module) => module.name === 'ctm');
      this.communeCheckedModules = this.checkedModulesFilter.filter((module) => module.name === 'commune');

      // this.updateQueryParams();
      this.applyFilter();
    }
  }
}
